import os
import nbformat


THIS_DIR = os.path.dirname(os.path.realpath(__file__))
FIXTURES_DIR = os.path.join(THIS_DIR, "fixtures")


def load_fixture_nb(name):
    with open(os.path.join(FIXTURES_DIR, name + ".ipynb")) as fp:
        return nbformat.read(fp, as_version=4)



from nbconvert.preprocessors import Preprocessor
from traitlets import Unicode, List


class AddAuthors(Preprocessor):
    """
    Add author names to each notebook.
    """
    authors = List(Unicode)

    def preprocess(self, nb, resources):
        if self.authors:
            authors = [{'name': author} for author in self.authors]
            nb['metadata']['authors'] = authors
        return nb, resources


class ElideCell(Exception):
    pass


def elide_fully(cell):
    raise ElideCell()


def clear_source(cell):
    cell['source'] = ''


def clear_outputs(cell):
    cell['outputs'] = []



class EmptyCellElider(Preprocessor):
    def preprocess(self, nb, resources):
        new_cells = []

        for cell in nb['cells']:
            if cell.get('source', '').strip():
                new_cells.append(cell)

        nb['cells'] = new_cells

        return nb, resources



class CellElider(Preprocessor):
    elide_fully_tags = List(Unicode(), default_value=['private',
                                                      'setup',
                                                      'todo'])
    clear_source_tags = List(Unicode(), default_value=['output-generator'])
    clear_output_tags = List(Unicode(), default_value=['run-verified-code'])

    def preprocess(self, nb, resources):
        rules = {}

        for k in self.elide_fully_tags:
            rules[k] = elide_fully

        for k in self.clear_source_tags:
            rules[k] = clear_source

        for k in self.clear_output_tags:
            rules[k] = clear_outputs

        new_cells = []

        for cell in nb['cells']:
            try:
                for tag in cell.get('metadata', {}).get('tags', []):
                    f = rules.get(tag)
                    if f:
                        f(cell)
                new_cells.append(cell)
            except ElideCell:
                pass

        nb['cells'] = new_cells

        return nb, resources



import unittest

from tests import load_fixture_nb
from dissertate.preprocessors import AddAuthors, EmptyCellElider, CellElider


class TestAddAuthors(unittest.TestCase):

    def test_add_authors(self):
        nb = load_fixture_nb("basic")
        f = AddAuthors()

        nb_, _ = f.preprocess(nb, [])
        self.assertNotIn('authors', nb_['metadata'])

        f.authors = ['Jack', 'Jill']
        nb_, _ = f.preprocess(nb, [])
        self.assertIn('authors', nb_['metadata'])
        self.assertEqual(nb_['metadata']['authors'],
                         [{'name': 'Jack'}, {'name': 'Jill'}])


class TestEmptyCellElider(unittest.TestCase):
    def test_empty_cell_elider(self):
        cells = [{'source': ' '},
                 {'source': 'A'},
                 {'source': 'B'},
                 {'nosource': '\t'}]

        nb, _ = EmptyCellElider().preprocess({'cells': cells}, None)
        source = [cell['source'] for cell in nb['cells']]
        self.assertEqual("".join(source), "AB")



class TestCellElider(unittest.TestCase):
    def test_cell_elider(self):
        cells = [{'source': 'a', 'outputs': ['A']},
                 {'source': 'b', 'outputs': ['B'],
                  'metadata': {'tags': ['private']}},
                 {'source': 'c', 'outputs': ['C'],
                  'metadata': {'tags': ['setup']}},
                 {'source': 'd', 'outputs': ['D'],
                  'metadata': {'tags': ['output-generator']}},
                 {'source': 'e', 'outputs': ['E'],
                  'metadata': {'tags': ['run-verified-code']}},
                 {'source': 'f', 'outputs': ['F']}]

        nb, _ = CellElider().preprocess({'cells': cells}, None)
        source, outputs = [], []
        for cell in nb['cells']:
            source.append(cell['source'])
            outputs.extend(cell['outputs'])
        self.assertEqual("".join(source), "aef")
        self.assertEqual("".join(outputs), "ADF")


if __name__ == '__main__':
    unittest.main()



from functools import partial
from nbformat import read as read_nb
from nbformat import write as write_nb

from dissertate.util import walk_paths


def transform(nb_path, *fs):
    with open(nb_path, "r") as fp:
        nb = read_nb(fp, as_version=4)

    for f in fs:
        f(nb)

    with open(nb_path, "w") as fp:
        write_nb(nb, nb_path)


def add_authors(nb, authors):
    nb['metadata']['authors'] = [{'name': author} for author in authors]


def update_private_meta(nb, namespace, meta_dict):
    nb['metadata'].update(meta_dict)


if __name__ == '__main__':
    working_title = "A Hacker's Guide to Heaven and Hell"

    print("Applying transformations...")
    for _, path, _ in walk_paths("."):
        if not path.lower().endswith(".ipynb"):
            continue
        print("Editing {}".format(path))
        transform(path,
                  partial(update_private_meta,
                          namespace='dissertate',
                          meta_dict={'title': working_title}),
                  partial(add_authors,
                          authors=['John Bjorn Nelson']))



__title__ = "dissertate"
__author__ = "John Bjorn Nelson (generativist)"
__email__ = "jbn@abreka.com"
__description__ = "A package and CLI for writing dissertations."
__uri__ = "https://github.com/jbn/dissertate"
__doc__ = __description__ + " <" + __uri__ + ">"
__license__ = "MIT"
__copyright__ = "Copyright (c) 2017 John Bjorn Nelson"
__version__ = "0.0.1"



import os


def walk_paths(base_dir, recursive=True, ignore_hidden=True,
               ignore_private=True):
    for file_name in os.listdir(base_dir):
        if ignore_hidden and file_name.startswith('.'):
            continue

        if ignore_private and file_name.startswith('_'):
            continue

        path = os.path.join(base_dir, file_name)

        if os.path.isdir(path):
            if recursive:
                for nb_path in walk_paths(path):
                    yield nb_path
        else:
            yield file_name, path, base_dir



import os
import bibtexparser
from dissertate.util import walk_paths


def walk_bibs(base_dir):
    for parts in walk_paths("."):
        if parts[0].lower().endswith('.bib'):
            yield parts


class Bibliography:
    def __init__(self, base_dir):
        self._root_dir = os.path.abspath(base_dir)
        self._bibs = {}

        for file_name, path, base_dir in walk_bibs("."):
            with open(path) as bibtex_file:
                bib = bibtexparser.load(bibtex_file)
            self._bibs[path] = bib

    @property
    def root_dir(self):
        return self._root_dir

    def iterate_entries(self):
        for bib_db in self._bibs.values():
            for entry in bib_db.entries:
                yield entry

    @property
    def all_entries(self):
        return list(self.iterate_entries())

    def simple_search(self, k, q, with_bibpath=True):
        q = q.lower()
        # See conventions for capitalization:
        # https://bibtexparser.readthedocs.io/en/v0.6.2/tutorial.html
        for path, bib_db in self._bibs.items():
            for entry in bib_db.entries:
                v = entry.get(k, '')
                if v and q in v.lower():
                    d = {k: v for k, v in entry.items()}
                    if with_bibpath:
                        full_path = os.path.join(self.root_dir, path)
                        d['bib_path'] = os.path.normpath(full_path)
                    yield d


if __name__ == '__main__':
    bib = Bibliography(os.getcwd())
    print("Scanning from:", bib.root_dir)
    for entry in bib.all_entries:
        print(entry)



import codecs
import os
import re

from setuptools import setup, find_packages

###############################################################################

NAME = 'dissertate'

PACKAGES = find_packages(where=".")

META_PATH = os.path.join("dissertate", "__init__.py")

KEYWORDS = ["phd", "dissertation", "thesis", "jupyter", "ipython"]

CLASSIFIERS = [
    "Development Status :: 3 - Alpha",
    "Framework :: IPython",
    "Intended Audience :: Science/Research",
    "Natural Language :: English",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2",
    "Programming Language :: Python :: 2.6",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Topic :: Software Development :: Libraries :: Python Modules",
]

INSTALL_REQUIRES = []

###############################################################################

SELF_DIR = os.path.abspath(os.path.dirname(__file__))


def read_file_safely(*path_parts):
    with codecs.open(os.path.join(SELF_DIR, *path_parts), "rb", "utf-8") as f:
        return f.read()


META_FILE = read_file_safely(META_PATH)

META_VARS_RE = re.compile(r"^__([_a-zA-Z0-9]+)__ = ['\"]([^'\"]*)['\"]", re.M)

META_VARS = dict(META_VARS_RE.findall(META_FILE))

###############################################################################

if __name__ == "__main__":
    setup(
        name=NAME,
        description=META_VARS["description"],
        license=META_VARS["license"],
        url=META_VARS["uri"],
        version=META_VARS["version"],
        author=META_VARS["author"],
        author_email=META_VARS["email"],
        maintainer=META_VARS["author"],
        maintainer_email=META_VARS["email"],
        keywords=KEYWORDS,
        long_description=read_file_safely("README.rst"),
        packages=PACKAGES,
        package_dir={"": "."},
        zip_safe=False,
        classifiers=CLASSIFIERS,
        install_requires=INSTALL_REQUIRES,
    )
